"""Initialization file for the `click_extended.core.other` module."""

from click_extended.core.other.context import Context
from click_extended.core.other.get_context import get_context

__all__ = [
    "Context",
    "get_context",
]
